<?php

declare(strict_types=1);

namespace Dedi\SyliusBlogPlugin\Entity;

use DateTime;
use Dedi\SyliusBlogPlugin\Domain\Blog\Adapter\AuthorInterface;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;
use Doctrine\ORM\Mapping as ORM;
use Sylius\Component\Core\Model\ChannelInterface;
use Sylius\Component\Core\Model\ImageInterface;
use Sylius\Component\Resource\Model\TranslationInterface;
use Symfony\Component\Validator\Constraints as Assert;

/**
 * @ORM\MappedSuperclass
 * @ORM\Table(name="dedi_blog_article")
 * @ORM\HasLifecycleCallbacks()
 */
class Article extends AbstractReferable implements ArticleInterface
{
    /**
     * @ORM\Column(type="integer", nullable=false)
     * @ORM\Id()
     * @ORM\GeneratedValue(strategy="AUTO")
     */
    private ?int $id = null;

    /** @ORM\Column(type="boolean", nullable=false, name="article_enabled") */
    protected bool $enabled = false;

    /**
     * @ORM\Column(type="integer", nullable=false, name="article_position")
     * @Assert\PositiveOrZero(groups={"default", "sylius"})
     */
    protected int $position = 0;

    /** @ORM\OneToOne(targetEntity="Dedi\SyliusBlogPlugin\Entity\ArticleBanner", mappedBy="owner", orphanRemoval=true, cascade={"persist"}) */
    private ?ImageInterface $banner = null;

    /**
     * @ORM\ManyToMany(targetEntity="Dedi\SyliusBlogPlugin\Entity\Category", inversedBy="articles")
     * @ORM\JoinTable(name="dedi_blog_articles_categories",
     *     joinColumns={@ORM\JoinColumn(name="article_id", referencedColumnName="id")},
     *     inverseJoinColumns={@ORM\JoinColumn(name="category_id", referencedColumnName="id")}
     * )
     */
    protected Collection $categories;

    /**
     * @ORM\ManyToMany(targetEntity="Sylius\Component\Core\Model\ChannelInterface")
     * @ORM\JoinTable(name="dedi_blog_articles_channels",
     *     joinColumns={@ORM\JoinColumn(name="article_id", referencedColumnName="id")},
     *     inverseJoinColumns={@ORM\JoinColumn(name="channel_id", referencedColumnName="id")}
     * )
     */
    protected Collection $channels;

    /**
     * @ORM\ManyToOne(targetEntity="Dedi\SyliusBlogPlugin\Entity\Author")
     * @ORM\JoinColumn(name="author_id", referencedColumnName="id", nullable=true)
     */
    private ?AuthorInterface $author = null;

    /** @ORM\Column(type="datetime", nullable=false, name="article_created_at") */
    private ?DateTime $createdAt;

    /** @ORM\Column(type="datetime", nullable=true, name="article_edited_at") */
    private ?DateTime $editedAt = null;

    /**
     * https://github.com/Sylius/SyliusResourceBundle/blob/master/src/Bundle/DependencyInjection/Compiler/DoctrineTargetEntitiesResolverPass.php
     *
     * @ORM\OneToMany(targetEntity="Dedi\SyliusBlogPlugin\Entity\CommentInterface", mappedBy="article", orphanRemoval=true, cascade={"remove", "persist"})
     */
    private Collection $comments;

    /**  @ORM\ManyToOne(targetEntity="Dedi\SyliusBlogPlugin\Entity\TypeArticle") */
    protected TypeArticleInterface $type;

    public function __construct()
    {
        parent::__construct();

        $this->categories = new ArrayCollection();
        $this->channels = new ArrayCollection();
        $this->comments = new ArrayCollection();
        $this->createdAt = new DateTime();
        $this->editedAt = new DateTime();
    }

    protected function createTranslation(): TranslationInterface
    {
        return new ArticleTranslation();
    }

    public function getId(): ?int
    {
        return $this->id;
    }

    public function getPosition(): int
    {
        return $this->position;
    }

    public function setPosition(int $position): self
    {
        $this->position = $position;

        return $this;
    }

    public function getBanner(): ?ImageInterface
    {
        return $this->banner;
    }

    public function setBanner(?ImageInterface $banner): self
    {
        $banner->setOwner($this);
        $this->banner = $banner;

        return $this;
    }

    public function getChannels(): Collection
    {
        return $this->channels;
    }

    public function setChannels(Collection $channels): self
    {
        $this->channels = $channels;

        return $this;
    }

    public function addChannel(ChannelInterface $channel): bool
    {
        if (!$this->channels->contains($channel)) {
            return $this->channels->add($channel);
        }

        return false;
    }

    public function removeChannel(ChannelInterface $channel): bool
    {
        return $this->channels->removeElement($channel);
    }

    public function isEnabled(): bool
    {
        return $this->enabled;
    }

    public function setEnabled(bool $enabled): self
    {
        $this->enabled = $enabled;

        return $this;
    }

    public function getSlug(): string
    {
        /** @var ArticleTranslationInterface $translation */
        $translation = $this->getTranslation();

        return $translation->getSlug();
    }

    public function setSlug(string $slug): self
    {
        /** @var ArticleTranslationInterface $translation */
        $translation = $this->getTranslation();
        $translation->setSlug($slug);

        return $this;
    }

    public function getTitle(): string
    {
        /** @var ArticleTranslationInterface $translation */
        $translation = $this->getTranslation();

        return $translation->getTitle();
    }

    public function setTitle(string $title): self
    {
        /** @var ArticleTranslationInterface $translation */
        $translation = $this->getTranslation();
        $translation->setTitle($title);

        return $this;
    }

    public function getContent(): string
    {
        /** @var ArticleTranslationInterface $translation */
        $translation = $this->getTranslation();

        return $translation->getContent();
    }

    public function setContent(string $content): self
    {
        /** @var ArticleTranslationInterface $translation */
        $translation = $this->getTranslation();
        $translation->setContent($content);

        return $this;
    }

    public function getCategories(): Collection
    {
        return $this->categories;
    }

    public function setCategories(Collection $categories): self
    {
        $this->categories = $categories;

        return $this;
    }

    public function hasCategories(): bool
    {
        return !$this->categories->isEmpty();
    }

    public function addCategory(CategoryInterface $category): bool
    {
        if (!$this->categories->contains($category)) {
            return $this->categories->add($category);
        }

        return false;
    }

    public function removeCategory(CategoryInterface $category): bool
    {
        return $this->categories->removeElement($category);
    }

    public function getAuthor(): ?AuthorInterface
    {
        return $this->author;
    }

    public function setAuthor(?AuthorInterface $author): self
    {
        $this->author = $author;

        return $this;
    }

    public function getCreatedAt(): DateTime
    {
        return $this->createdAt;
    }

    public function getEditedAt(): DateTime
    {
        return $this->editedAt;
    }

    public function getMetadataTitle(): ?string
    {
        return $this->getTranslation()->getMetadataTitle() ?? $this->getTitle();
    }

    public function getMetadataDescription(): ?string
    {
        return $this->getTranslation()->getMetadataDescription() ?? substr(strip_tags($this->getContent()), 0, 155);
    }

    public function getTranslation(?string $locale = null): AbstractReferableTranslation
    {
        /** @var ArticleTranslation $translation */
        $translation = parent::getTranslation($locale);

        return $translation;
    }

    public function getDescription(): ?string
    {
        /** @var ArticleTranslationInterface $translation */
        $translation = $this->getTranslation();

        return $translation->getDescription();
    }

    public function setDescription(?string $description): self
    {
        /** @var ArticleTranslationInterface $translation */
        $translation = $this->getTranslation();
        $translation->setDescription($description);

        return $this;
    }

    /**
     * @ORM\PrePersist()
     */
    public function updateCreatedAt(): void
    {
        $this->createdAt = new DateTime();
    }

    /**
     * @ORM\PreUpdate()
     */
    public function updateEditedAt(): void
    {
        $this->editedAt = new DateTime();
    }

    public function getComments(): Collection
    {
        return $this->comments;
    }

    public function addComment(CommentInterface $comment): self
    {
        $this->comments[] = $comment;

        return $this;
    }

    public function __toString(): string
    {
        return $this->getTitle();
    }

    public function getType(): TypeArticleInterface
    {
        return $this->type;
    }

    public function setType(TypeArticleInterface $type): self
    {
        $this->type = $type;

        return $this;
    }
}
